using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;
using PrefabEvolution;

// WhiteRoomEditor v1.0
//
// Allows you to edit some parameters of Force Field 01

namespace WhiteRoomEditorPlugin
{
    public class WhiteRoomEditor : MVRScript
    {
		public JSONStorableColor RoomColor;
		
		public Transform ObjectRoot;
		private Renderer objRenderer;
		private string objectSuffixName = "room_base";

		private bool seInitialized = false;
		
		// Security
		private int initLoops = 0;
		private int maxInitLoops = 3600;

		
		public override void Init()
        {			 
            try
            {
				if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("Please add WhiteRoomEditor on an Custom Unity Asset with a compatible object.");
                    return;
                }
				
				HSVColor hsvc = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
				RoomColor = new JSONStorableColor("ForceField color", hsvc, RoomColorCallback);
				RegisterColor(RoomColor);
				CreateColorPicker(RoomColor);
										
				JSONStorableString helpText = new JSONStorableString("Help",
					"<color=#000><size=35><b>Help</b></size></color>\n\n" + 
					"<color=#333>" +
					"<b>WhiteRoomEditor</b> is a tiny plugin that allows you to edit the color of the white-room-lit and white-room-unlit in this package\n\n" +
					"Change the color on the left, the room will update in realtime.\n\n" +
					"<b>IMPORTANT :</b> If you switch objects in the Asset tab of the CUA, don't forget to reload the script to enable customization.\n\n" +
					"</color>"
				);
				UIDynamic helpTextfield = CreateTextField(helpText, true);
				helpTextfield.height = 1100.0f;
				

			}
            catch(Exception e)
            {
                SuperController.LogError("WhiteRoomEditor - Exception caught: " + e);
            }
		}
		
		void Start()
        {

        }
		
		void Update()
        {
			// If we haven't made over 1min of init tests, we aren't initialized, try to initialize
			if( initLoops < maxInitLoops && seInitialized == false ) {
				initLoops++;
				InitEditor();
			}
			
			if( initLoops == maxInitLoops ) {
				logError("WhiteRoomEditor : This CUA could not be initialized properly after 3600 tries. Please select a compatible object and reload the script.");
				initLoops++; // To avoid spamming the log
			}
		}
		
		private bool InitEditor() {
			if( seInitialized == true ) return true;
			
			// Thanks for the tip meshed !
			ObjectRoot = getObjectRoot( containingAtom.reParentObject );
			if( ObjectRoot == null ) {
				return false;
			}
			
			objRenderer = ObjectRoot.GetComponent<Renderer>();
			if( objRenderer ) {
				seInitialized = true;
				
				RoomColorCallback( RoomColor );
							
				logDebug("WhiteRoomEditor : White room editor initialized properly");
							
				return true;
			} else {
				logError("WhiteRoomEditor : your CUA has a transform with the good name but does not have a renderer.");
				initLoops = maxInitLoops; // We don't have a particle system, we consider the initialization failed
				return false;
			}
		}
		
		private Transform getObjectRoot( Transform parent ) {
			foreach( Transform child in parent )
			{
				if( child.name.Contains(objectSuffixName) ) {
					return child;
				} else {
					Transform childSearch = getObjectRoot(child);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}
		
		protected void RoomColorCallback(JSONStorableColor color) {
			if( objRenderer ) {
				Color newColor = new Color(color.colorPicker.currentColor.r, color.colorPicker.currentColor.g, color.colorPicker.currentColor.b, 1.0f);
				objRenderer.material.SetColor("_Color",newColor);
			}
		}

		
		// **************************
		// Local Tools
		// **************************
		private void logDebug( string debugText ) {
			SuperController.LogMessage( debugText );
		}
		
		private void logError( string debugText ) {
			SuperController.LogError( debugText );
		}
	}
}